<div>
    <div class="cms_tooltip">
        <div>Click to edit</div>
        <span style="float: right;"></span>
    </div>
    <div class="cms_toolbar_snippet">
        Edit this page
    </div>
    <div class="cms_toolbar">
        <ul class="cms_toolbar_list">
            <li>
                <button id="cms_toolbar_hide">
                    Hide >>
                </button>
                <button id="save_content">Save as draft</button>
                <button id="publish">Publish</button>
            </li>
        </ul>
    </div>
    <div class="cms_editor">
        <div class="cms_close" id="close">&times;</div>
        <div id="editors"></div>
    </div>
</div>
