<!DOCTYPE html>
<html lang="en">
<head>
    <meta charset="UTF-8">
    {% block head %}
        <title>Page Title</title>
    {% endblock %}
    {% block meta %}
    {% endblock %}
    {% block head_css %}
    {% endblock %}
    {% block head_js %}
    {% endblock %}
</head>
<body>
{% block content %}

{% endblock %}
{% ifequal view 'author' %}
{% include 'cfblog_admin/toolbar.html' %}
{% include 'cfblog_admin/script.html' %}
{% endifequal %}
{% block footer_js %}

{% endblock %}
</body>
</html>
