<style>
.cms_tooltip {
    visibility: hidden;
    position: absolute;
    -webkit-border-radius: 3px;
    font-size: 12px;
    line-height: 11px;
    white-space: nowrap;
    margin: 0;
    float: right;
    -moz-border-radius: 3px 0 3px 3px;
    border-radius: 3px 0 3px 3px;
    color: white;
    font-weight: normal;
    padding: 5px 7px;
    background: #454545;
}
.cms_toolbar {
    background-color: rgba(34, 34, 34, 0.9);
    color: whitesmoke;
    position: fixed;
    width: 20%;
    top: 0;
    bottom: 0;
    right: 0;
    z-index: 100000000000;
    overflow-y: auto;
}
.cms_toolbar div {
    cursor: pointer;
}
.cms_toolbar_list {
    list-style-type: none;
    padding: 0;
    margin: 0;
}
.cms_toolbar_list li {
    padding: 4%;
    border-bottom: 1px solid #fff;
}
.cms_editor {
    background-color: rgb(255, 255, 255);
    width: 80%;
    visibility: hidden;
    position: fixed;
    top: 0;
    bottom: 0;
    left: 0;
    z-index: 100000000000;
    overflow-y: auto;
}
.cms_editor .cms_close {
    top: 0;
    color: #000;
    float: right;
    text-align: center;
    width: 20px;
    height: 20px;
    border-radius: 50%;
    background-color: #ffed09;
}
.cms_editor .cms_close:hover {
    cursor: pointer;
}
.cms_toolbar_snippet {
    font-style: oblique;
    cursor: pointer;
    visibility: hidden;
    height: 20px;
    padding: 10px;
    background-color: #311705;
    color: whitesmoke;
    float: right;
    position: absolute;
    z-index: 100000000000;
    top: 35%;
    right: 0;
    -webkit-transform: rotate(-90deg);
    -moz-transform: rotate(-90deg);
    -ms-transform: rotate(-90deg);
    -o-transform: rotate(-90deg);
    transform-origin: right bottom;
}
.cms_highlight {
    background-color: whitesmoke;
    color: black;
}
</style>
<script type="text/javascript">
    var draft_url = '{% url 'save_cms_content' 'draft' %}';
    var publish_url = '{% url 'save_cms_content' 'publish' %}';
    var CSRF_TOKEN = '{{ csrf_token }}';
    var cms_data = {{ cms_data|safe }};
    var namespace_delimiter = {{ namespace_delimiter|safe }};
    var local_storage_name = '{{ request.path|slugify }}';
    var cms_page_id = '{{ id }}';
    var static_url = '{{ STATIC_URL }}' + 'cfblog/';
</script>
<script src="{{ STATIC_URL }}cfblog/js/jquery-1.7.1.min.js"></script>
<script src="{{ STATIC_URL }}cfblog/js/epiceditor.min.js"></script>
<script src="{{ STATIC_URL }}cfblog/js/edit.js"></script>
